using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.HttpsPolicy;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;

namespace ScrumTaskboard
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {   
            services.AddCors(o => o.AddPolicy("AllowAll", builder =>
            {
                builder.AllowAnyOrigin()
                       .AllowAnyMethod()
                       .AllowAnyHeader();
            }));
            services.AddScoped(serviceProvider => new TaskContext(
                new DbContextOptionsBuilder<TaskContext>()
                .UseNpgsql(GetConnectionString())
                .Options));
            services.AddControllers();
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            using (var serviceScope = app.ApplicationServices.GetService<IServiceScopeFactory>().CreateScope())
            {
                var context = serviceScope.ServiceProvider.GetRequiredService<TaskContext>();
                context.Database.EnsureCreated();
            }

            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            //app.UseHttpsRedirection();
            app.UseCors("AllowAll");

            app.UseRouting();

            app.UseAuthorization();

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });
        }

        /// <summary>
        /// Creates DB Connection String based on ENV vars and default vars.
        /// </summary>
        /// <returns></returns>
        public string GetConnectionString()
        {
            string dbHost;
            string dbPort;
            string dbName;
            string dbUser;
            string dbPassword;

            dbHost = Environment.GetEnvironmentVariable("DATABASE_HOST");
            dbPort = Environment.GetEnvironmentVariable("DATABASE_PORT");
            dbName = Environment.GetEnvironmentVariable("DATABASE_NAME");
            dbUser = Environment.GetEnvironmentVariable("DATABASE_USER");
            dbPassword = Environment.GetEnvironmentVariable("DATABASE_PASSWORD");

            if(dbHost == null || dbPort == null || dbName == null || dbUser == null || dbPassword == null)
            {
                dbHost = "nig.gl";
                dbPort = "8543";
                dbName = "taskboard";
                dbUser = "scrum";
                dbPassword = "c6gXud7YvBWp2sgxSgy4wRN";
            }
            

            return $"Host={dbHost}; Port={dbPort}; Username={dbUser}; Database={dbName}; Password={dbPassword}";
            }
    }
}
